import os
import vot
import numpy as np
from sbbm_complete import SBBM

# vot framework sets cwd to a temp directory but we need to read settings.ini 
# from same folder python script is in, so change cwd to script folder
os.chdir(os.path.dirname(__file__))

# start vot framework
handle = vot.VOT("polygon")

# read in first frame and bounding box
frame_path = handle.frame()
bbox = np.array(handle.region().points).ravel()

# initialise tracker
tracker = SBBM(frame_path, bbox)

# process frames until the framework does not give us another one
while True:
    frame_path = handle.frame()
    
    if not frame_path:
        break

    # track object and get predicted bbox and patch likelihoods
    bb, _, patch_confidence = tracker.process_frame(frame_path)
    
    # convert to vot region and average patch likelihood
    region = vot.Polygon([vot.Point(x, y) for x, y in zip(bb[::2], bb[1::2])])
    confidence = np.mean(patch_confidence)
    
    # report to framework
    handle.report(region, confidence)
